/**
 * <b>Assessment</b> part three<br>
 * <b>Created</b> 08/11/18<br>
 * <b>Modified</b> 04/01/18<br>
 * @author <a href = "mailto:neil.whitehead@yorksj.ac.uk"> Neil Whitehead</a>, <a href = "mailto:a.guest@yorksj.ac.uk">Andrew Guest</a>
 */

import java.util.ArrayList;
import java.util.List;
import java.io.*;

public class AssessmentPartFour 
{	
	List<String> morseCode = new ArrayList<String>();
	//global variable
	
	/**
	 * Populates list with input from text file <br>
	 * Assumes that no validation checks are required.
	 * @param filename string
	 * @param morseCode string
	 * @return morseCode.size int
	 */
	public int loadMorseFromFile(String filename)
	{
		morseCode = new ArrayList<String>();
	
		try (FileReader file = new FileReader (filename); BufferedReader buffer = new BufferedReader (file))
		//try/catch blocks will ensure that the program can recover from errors
		//creating new file class so program can interact with file
		//creating new buffer, which is faster to read from than repeatedly accessing the file directly
		{
			String strLine = "";
		
			while ((strLine = buffer.readLine()) !=null)
			//whilst there is something to add. . .
			{
				morseCode.add(strLine);
				//. . . add to list
			}
		//buffer.close(); not needed as try-with-resources will automatically close
		}
		catch (IOException e)
		//only used when error occurs.
		{
			System.out.println("Exception: " + e.getMessage());
			//e.getMessage retrieves a string with an error message to further explain issue
		}
		return morseCode.size();
	}
	
	
	/**
	 * Translates morse into standard English <br>
	 * Assumes that no validation checks are required.
	 * @param morseCode string array list (global variable)
	 * @return strTranslated string
	 */
	public String translateMorse()
	{	
		String strTranslated = "";
		
		String[][] strMorseArray = 
				//morse-letter lookup table, segregated by morse code length
				//nb length corresponds with frequency within English language, so perhaps quicker than alphabetical
		/* 0 */	{{" ", "E", "T"}, // have 1 char
		/* 1 */	 {"/", ".", "-"},
		/* 2 */	 {"A", "I", "N", "M"}, // have 2 char
		/* 3 */	 {".-", "..", "-.", "--"},
		/* 4 */	 {"D", "G", "K", "O", "R", "S", "U", "W"}, // have 3 char
		/* 5 */	 {"-..", "--.", "-.-", "---", ".-.", "...", "..-", ".--"},
		/* 6 */	 {"B", "C", "F", "H", "J", "L", "P", "Q", "V", "X", "Y", "Z"}, // have 4 char
		/* 7 */	 {"-...", "-.-.", "..-.", "....", ".---", ".-..", ".--.", "--.-", "...-", "-..-", "-.--", "--.."},
		/* 8 */	 {"1", "2", "3", "4", "5", "6", "7", "8", "9", "0"}, // have 5 char
		/* 9 */	 {".----", "..---", "...--", "....-", ".....", "-....", "--...", "---..", "----.", "-----"}};
		
		for(int i = 0; i < morseCode.size(); i++)
		//loops through each morse word within list (which has been taken from file)
		{
			String strUntranslated = morseCode.get(i);
			
			int x = (((strUntranslated).length()) * 2) - 1;
			//length of the morse input, times 2, minus 1 is the x co-ordinate of morse code within the array.
			
			for(int y = 0; y < strMorseArray[x].length; y++)
			//loop through to identify y co-ordinate
			{	
				if(strMorseArray[x][y].contains(strUntranslated))
				//contains runs fractionally faster than equals.
				//only checks against morse code which is the same length, so ".." containing "." is not an issue.
				{					
					strTranslated = strTranslated.concat(strMorseArray[x-1][y]);
					//x minus one = corresponding translation of morse code within the array
					//adding recently matched letter to translated string
					break;
				}
			}	
		}
		strTranslated = strTranslated.toLowerCase();
		//converting to lowercase to comply with unit test
		return strTranslated;
	}

}
